settings = dict(
    provider = 'sqlite', filename = ':memory:'
    # provider='postgres', user='pony', password='pony', host='localhost', port='5432', database='pony',
    # provider='cockroach', user='root', host='localhost', port=26257, sslmode='disable', database='test'
)
